@@ -43,7 +43,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
     """Connector class to attach/detach iSCSI volumes."""
 
     supported_transports = ['be2iscsi', 'bnx2i', 'cxgb3i', 'default',
-                            'cxgb4i', 'qla4xxx', 'ocs', 'iser']
+                            'cxgb4i', 'qla4xxx', 'ocs', 'iser', 'tcp']
     VALID_SESSIONS_PREFIX = ('tcp:', 'iser:')
 
     def __init__(self, root_helper, driver=None,
@@ -268,7 +268,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         """Check that given iscsi_iface uses only supported transports
 
         Accepted transport names for provided iface param are
-        be2iscsi, bnx2i, cxgb3i, cxgb4i, default, qla4xxx, ocs or iser.
+        be2iscsi, bnx2i, cxgb3i, cxgb4i, default, qla4xxx, ocs, iser or tcp.
         Note the difference between transport and iface;
         unlike default(iscsi_tcp)/iser, this is not one and the same for
         offloaded transports, where the default format is
@@ -845,10 +845,12 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         :type ignore_errors: bool
         """
         return self._cleanup_connection(connection_properties, force=force,
-                                        ignore_errors=ignore_errors)
+                                        ignore_errors=ignore_errors,
+                                        device_info=device_info)
 
     def _cleanup_connection(self, connection_properties, ips_iqns_luns=None,
-                            force=False, ignore_errors=False):
+                            force=False, ignore_errors=False,
+                            device_info=None):
         """Cleans up connection flushing and removing devices and multipath.
 
         :param connection_properties: The dictionary that describes all
@@ -864,6 +866,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         :param ignore_errors: When force is True, this will decide whether to
                               ignore errors or raise an exception once finished
                               the operation.  Default is False.
+        :param device_info: Attached device information.
         :type ignore_errors: bool
         """
         exc = exception.ExceptionChainer()
@@ -880,9 +883,14 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         remove_devices = set()
         for remove, __ in devices_map.values():
             remove_devices.update(remove)
-        multipath_name = self._linuxscsi.remove_connection(remove_devices,
-                                                           self.use_multipath,
-                                                           force, exc)
+
+        path_used = self._linuxscsi.get_dev_path(connection_properties,
+                                                 device_info)
+        was_multipath = (path_used.startswith('/dev/dm-') or
+                         'mpath' in path_used)
+        multipath_name = self._linuxscsi.remove_connection(
+            remove_devices, self.use_multipath, force, exc,
+            path_used, was_multipath)
 
         # Disconnect sessions and remove nodes that are left without devices
         disconnect = [conn for conn, (__, keep) in devices_map.items()
@@ -1102,9 +1110,12 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         return (out, err)
 
     def _get_node_startup_values(self, connection_properties):
+        # Exit code 21 (ISCSI_ERR_NO_OBJS_FOUND) occurs when no nodes
+        # exist - must consider this an empty (successful) result.
         out, __ = self._run_iscsiadm_bare(
             ['-m', 'node', '--op', 'show', '-p',
-             connection_properties['target_portal']]) or ""
+             connection_properties['target_portal']],
+            check_exit_code=(0, 21)) or ""
         node_values = out.strip()
         node_values = node_values.split("\n")
         iqn = None
